package Leonardo;
//Leonardo.java

/* Author Michael Claudius, Roskilde Computer Science College
 * Version 1.0. 2004.01.28
 * Copyright 2004 by Michael Claudius
 * All rights reserved ?
 */
import java.io.*;

public class Leonardo {

// constructor is empty
    Leonardo() {
    }

// defensive programming takes care of extreme cases like n < 0!
// recursive solution
    public int len(int n) {
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            return len(n - 1) + len(n - 2) + len(n - 3);
        }
    }

    public int lenWrong(int n) {
        System.out.println("called with " + n);
        if (n == 0 || n == 1 || n == 2) {
            return 1;
        }
        return lenWrong(n - 1) + len(n - 2) + len(n - 3);
    }

// iterative solution
    public int lenIt(int n) {
        int sum = 0;
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            int t1 = 1;
            int t2 = 1;
            int t3 = 1;
            for (int i = 3; i <= n; i++) {
                sum = t1 + t2 + t3;
                t3 = t2;
                t2 = t1;
                t1 = sum;
            }
            return sum;
        }
    }

// iterative solution saving in array
    public int lenItArray(int n) {
        int[] t = new int[n + 1];
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            t[0] = 1;
            t[1] = 1;
            t[2] = 1;
            for (int i = 3; i <= n; i++) {
                t[i] = t[i - 1] + t[i - 2] + t[i - 3];
            }
            return t[n];
        }
    }

// iterative solution saving in array returning whole array
    public int[] leonardoTable(int n) {
        // pre condition n>= 0;
        // post condition leonardo numbers 0-n in array t
        int[] t = new int[n + 1];  //n+1 is the length
        for (int i = 0; i <= n; i++) {
            t[i] = 1;
        }
        if (n > 2) {
            for (int i = 3; i <= n; i++) {
                t[i] = t[i - 1] + t[i - 2] + t[i - 3];
            }
        }


        return t;
    }

    public static void main(String[] args) {
        System.out.println("State number: ");

        int n = Keyboard.readInt();

        Leonardo myLen = new Leonardo();
        System.out.println("Recursive solution");
        System.out.println(myLen.len(n));

        System.out.println("Iterative solution");
        System.out.println(myLen.lenIt(n));

        System.out.println("Array solution");
        System.out.println(myLen.lenItArray(n));

        for (int i = 0; i <= n; i++) {
            System.out.println(myLen.lenIt(i));
        }

        // Faster as each number is only calculated once !
        int[] numbers = myLen.leonardoTable(n);
        for (int i = 0; i <= n; i++) {
            System.out.println(numbers[i]);
        }

    }
}
