package Leonardo;
//Leonardo.java

/* Author Michael Claudius, Roskilde Computer Science College
 * Version 1.0. 2004.01.28
 * Copyright 2004 by Michael Claudius
 * All rights reserved ?
 */
import java.io.*;

public class LeonardoStatic {

// constructor is empty
    LeonardoStatic() {
    }

// defensive programming takes care of extreme cases like n < 0!
// recursive solution
    public static int len(int n) {
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            return len(n - 1) + len(n - 2) + len(n - 3);
        }
    }

// iterative solution
    public static int lenIt(int n) {
        int lenNumber = 0;
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            int t1 = 1;
            int t2 = 1;
            int t3 = 1;
            for (int i = 3; i <= n; i++) {
                lenNumber = t1 + t2 + t3;
                t3 = t2;
                t2 = t1;
                t1 = lenNumber;
            }
            return lenNumber;
        }
    }

// iterative solution
    public static int lenItArray(int n) {
        int[] t = new int[n + 1]; //n+1 is the length
        if (n < 0) {
            return -1;
        } else if (n < 3) {
            return 1;
        } else {
            t[0] = 1;
            t[1] = 1;
            t[2] = 1;
            for (int i = 3; i <= n; i++) {
                t[i] = t[i - 1] + t[i - 2] + t[i - 3];
            }
            return t[n];
        }
    }

// iterative solution
    public static int[] leonardoTable(int n) {
        // pre condition n>= 0;
        // post condition leonardo numbers 0-n in array t
        int[] t = new int[n + 1]; //n+1 is the length
        for (int i = 0; i < 3; i++) {
            t[i] = 1;
        }
        if (n > 2) {
            for (int i = 3; i <= n; i++) {
                t[i] = t[i - 1] + t[i - 2] + t[i - 3];
            }
        }


        return t;
    }

    public static void main(String[] args) {
        System.out.println("State number: ");
        int n = Keyboard.readInt();

        System.out.println("Recursive solution");
        System.out.println(LeonardoStatic.len(n));
        System.out.println("Iterative solution");
        System.out.println(LeonardoStatic.lenIt(n));
        System.out.println("Array solution");
        System.out.println(LeonardoStatic.lenItArray(n));

        for (int i = 0; i <= n; i++) {
            System.out.println(LeonardoStatic.lenIt(i));
        }

        // Faster as each number is only calculated once !
        System.out.println("Whole table solution");
        int[] numbers = LeonardoStatic.leonardoTable(n);
        for (int i = 0; i <= n; i++) {
            System.out.println(numbers[i]);
        }

    }
}// Leonardo

